library(yaml)
args <- commandArgs(trailingOnly = TRUE)
params <- read_yaml(args[1])
params <- read_yaml("/esarchive/scratch/molmo/output/recipe_20230913_140034/run/diagnostic_1/script1/settings.yml")

plot_dir <- params$plot_dir
run_dir <- params$run_dir
work_dir <- params$work_dir

## Create working dirs if they do not exist
dir.create(plot_dir, recursive = TRUE)
dir.create(run_dir, recursive = TRUE)
dir.create(work_dir, recursive = TRUE)

library(ncdf4)
metadata1 <- read_yaml(params$input_files[1])

nc <- nc_open(metadata1$`/esarchive/scratch/molmo/output/recipe_20230913_140034/preproc/diagnostic_1/psl_d1/CMIP6_ACCESS-CM2_day_historical_r1i1p1f1_psl_1950-2014.nc`$filename)
gcm <-

#Directory information
setwd("/esarchive/scratch/molmo/")
OUTPUTS <- "/esarchive/scratch/molmo/output/"

#Useful functions
png.save <- function(x,filename,main,xlab,ylab){
  png(filename = paste(filename,".png",sep=""))
  plot(x,type="o",main=main,xlab = xlab,ylab=ylab)
  dev.off()
}

################################################################################
#Load SLP data
message("Loading and preprocessing data...")
nc_lon <- ncvar_get(nc,"lon")
nc_lat <- ncvar_get(nc,"lat")
nc_var <- ncvar_get(nc,"unknown") #x,y,time

#Selection of spatial domains
gridsize_x <- abs(nc_lon[2]-nc_lon[1])
gridsize_y <- abs(nc_lat[2]-nc_lat[1])

domain1 <- list(lon <- which(nc_lon %in% seq(-15,15,by=gridsize_x)),
               lat <- which(nc_lat %in% seq(32,48,by=gridsize_y))) #red domain D1 #Spain

domain2 <- list(lon <- which(nc_lon %in% seq(-10,25,by=gridsize_x)),
                lat <- which(nc_lat %in% seq(33,45,by=gridsize_y))) #red domain D2 PEP Mediterranean

domain3 <- list(lon <- which(nc_lon %in% seq(-35,35,by=gridsize_x)),
                lat <- which(nc_lat %in% seq(20,70,by=gridsize_y))) #red domain D3 #Europe

domain <- domain2 ; dom = "D2_1x1"
gc()

#Organize and Standardize data
var <- matrix(nc_var,dim(nc_var)[1]*dim(nc_var)[2],dim(nc_var)[3]) #space,time
rm(nc_var) ; gc()

################################################################################
#Data reduction
message("Data reduction by means of EOFs...")
RR=cor(var) ; gc()
DD=svd(RR)

U=DD$u ; V=DD$v
dv=DD$d ; dd=diag(DD$d)

FF=U%*%dd^0.5 #loadings matrix
A=dv^(-0.5)
AA=diag(A)
Zs=var.sc%*%U%*%AA  #CPs matrix

SumDiag=sum(dv)
VAR=(dv/SumDiag)*100 #explained variance

k=which(cumsum(VAR)>90)[1]
rm(RR,DD,U,V,A,AA,dd,FF) ; gc()
message("Done.")

################################################################################
#Clustering procedure
message("Performing clustering...")
input.data <- Zs[,1:k]
rm(Zs)

ward_clustering <- hclust(d = dist(input.data),method = "ward.D2")
clusters <- cutree(ward_clustering,k=5) #D2

save.image(file=paste(OUTPUTS,"WardClustering_D2.5_",dom,gcm,".Rdata",sep=""))

################################################################################
calendario <- seq(as.Date("1950-01-01"),as.Date("2014-12-31"),by="day")
fechas <- calendario[format(calendario,"%m-%d") != "02-29"]
n_years <- length(fechas)/365

if(length(clusters)!=length(fechas)){ #13149
  clusters <- clusters[which(calendario%in%fechas)]
  rm(calendario,fechas)
}

datos = data.frame(group=as.numeric(clusters),
                   day=rep(1:365,n_years))

value <- matrix(NA,365,k)
for(i in 1:365){
  for(j in 1:k){
    value[i,j] <- length(which(datos$group==j & datos$day==i))/n_years*100
  }
}

palette <- brewer.pal(n=7,name = "Set1")

datos = data.frame(day=rep(1:365,k),frequency=as.numeric(value),group=factor(rep(1:k,each=365),levels=c(1:k)))
p <- ggplot(data=datos,aes(x=day,y=frequency,fill=group)) + geom_col() + 
  scale_fill_manual(values=palette[1:k]) + theme_bw() + ggtitle("") + coord_cartesian(expand=F)+
  scale_x_continuous(labels = c("J","F","M","A","M","J","J","A","S","O","N","D"),breaks = seq(15,365,by=30))
ggsave(p,filename=paste(OUTPUTS,"relativefrequency_D2.5_CPs_",dom,"_",gcm,"_CPs.png",sep=""),device="png",
       dpi=600,width=4.84,height = 3.95)
